@@ -1,6 +1,6 @@
 /*
 Gwenview: an image viewer
-Copyright 2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2007 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,13 +30,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <kfileitem.h>
 #include <klocale.h>
 #include <ksqueezedtextlabel.h>
-#include <kwordwrap.h>
 
 // Local
 #include "contextmanager.h"
 #include "imagemetainfodialog.h"
 #include "sidebar.h"
 #include <lib/archiveutils.h>
+#include <lib/eventwatcher.h>
 #include <lib/gwenviewconfig.h>
 #include <lib/preferredimagemetainfomodel.h>
 #include <lib/document/document.h>
@@ -62,7 +62,6 @@ public:
 	KeyValueWidget(QWidget* parent)
 	: QWidget(parent)
 	, mLayout(new QFormLayout(this)) {
-		mLayout->setLabelAlignment(Qt::AlignRight);
 		setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
 	}
 
@@ -71,13 +70,12 @@ public:
 				"@item:intext %1 is a key, we append a colon to it. A value is displayed after",
 				"%1:", key);
 
-		QLabel* keyLabel = new QLabel;
+		KSqueezedTextLabel* keyLabel = new KSqueezedTextLabel;
+		keyLabel->setTextElideMode(Qt::ElideMiddle);
 		keyLabel->setText(keyString);
 
-		KSqueezedTextLabel* valueLabel = new KSqueezedTextLabel;
-		valueLabel->setTextElideMode(Qt::ElideRight);
+		QLabel* valueLabel = new QLabel;
 		valueLabel->setText(value);
-		valueLabel->setFocusPolicy(Qt::NoFocus);
 
 		mLayout->addRow(keyLabel, valueLabel);
 	}
@@ -94,7 +92,7 @@ private:
 
 
 struct InfoContextManagerItemPrivate {
-	SideBar* mSideBar;
+	InfoContextManagerItem* q;
 	SideBarGroup* mGroup;
 
 	// One selection fields
@@ -115,13 +113,43 @@ struct InfoContextManagerItemPrivate {
 		ImageMetaInfoModel* model = mDocument ? mDocument->metaInfo() : 0;
 		mImageMetaInfoDialog->setMetaInfo(model, GwenviewConfig::preferredMetaInfoKeyList());
 	}
+
+
+	void setupGroup() {
+		mOneFileWidget = new QWidget();
+
+		mKeyValueWidget = new KeyValueWidget(mOneFileWidget);
+
+		QLabel* moreLabel = new QLabel(mOneFileWidget);
+		moreLabel->setText(QString("<a href='#'>%1</a>").arg(i18nc("@action show more image meta info", "More...")));
+		moreLabel->setAlignment(Qt::AlignRight);
+
+		QVBoxLayout* layout = new QVBoxLayout(mOneFileWidget);
+		layout->setMargin(2);
+		layout->setSpacing(2);
+		layout->addWidget(mKeyValueWidget);
+		layout->addWidget(moreLabel);
+
+		mMultipleFilesLabel = new QLabel();
+
+		mGroup = new SideBarGroup(i18nc("@title:group", "Meta Information"));
+		q->setWidget(mGroup);
+		mGroup->addWidget(mOneFileWidget);
+		mGroup->addWidget(mMultipleFilesLabel);
+
+		EventWatcher::install(mGroup, QEvent::Show, q, SLOT(updateSideBarContent()));
+
+		QObject::connect(moreLabel, SIGNAL(linkActivated(const QString&)),
+			q, SLOT(showMetaInfoDialog()) );
+	}
 };
 
 
 InfoContextManagerItem::InfoContextManagerItem(ContextManager* manager)
 : AbstractContextManagerItem(manager)
 , d(new InfoContextManagerItemPrivate) {
-	d->mSideBar = 0;
+	d->q = this;
+	d->setupGroup();
 	connect(contextManager(), SIGNAL(selectionChanged()),
 		SLOT(updateSideBarContent()) );
 }
@@ -131,39 +159,9 @@ InfoContextManagerItem::~InfoContextManagerItem() {
 }
 
 
-void InfoContextManagerItem::setSideBar(SideBar* sideBar) {
-	d->mSideBar = sideBar;
-	connect(sideBar, SIGNAL(aboutToShow()),
-		SLOT(updateSideBarContent()) );
-	d->mOneFileWidget = new QWidget();
-
-	d->mKeyValueWidget = new KeyValueWidget(d->mOneFileWidget);
-
-	QLabel* moreLabel = new QLabel(d->mOneFileWidget);
-	moreLabel->setText(QString("<a href='#'>%1</a>").arg(i18nc("@action show more image meta info", "More...")));
-	moreLabel->setAlignment(Qt::AlignRight);
-
-	QVBoxLayout* layout = new QVBoxLayout(d->mOneFileWidget);
-	layout->setMargin(2);
-	layout->setSpacing(2);
-	layout->addWidget(d->mKeyValueWidget);
-	layout->addWidget(moreLabel);
-
-	d->mMultipleFilesLabel = new QLabel();
-
-	d->mGroup = sideBar->createGroup(i18nc("@title:group", "Meta Information"));
-	d->mGroup->addWidget(d->mOneFileWidget);
-	d->mGroup->addWidget(d->mMultipleFilesLabel);
-
-	d->mGroup->hide();
-
-	connect(moreLabel, SIGNAL(linkActivated(const QString&)), SLOT(showMetaInfoDialog()) );
-}
-
-
 void InfoContextManagerItem::updateSideBarContent() {
 	LOG("updateSideBarContent");
-	if (!d->mSideBar->isVisible()) {
+	if (!d->mGroup->isVisible()) {
 		LOG("updateSideBarContent: not visible, not updating");
 		return;
 	}
@@ -171,14 +169,12 @@ void InfoContextManagerItem::updateSideBarContent() {
 
 	KFileItemList itemList = contextManager()->selection();
 	if (itemList.count() == 0) {
-		d->mGroup->hide();
 		// "Garbage collect" document
 		d->mDocument = 0;
 		d->updateMetaInfoDialog();
 		return;
 	}
 
-	d->mGroup->show();
 	KFileItem item = itemList.first();
 	if (itemList.count() == 1 && !ArchiveUtils::fileItemIsDirOrArchive(item)) {
 		fillOneFileGroup(item);
@@ -215,11 +211,12 @@ void InfoContextManagerItem::fillMultipleItemsGroup(const KFileItemList& itemLis
 	}
 
 	if (folderCount == 0) {
-		d->mMultipleFilesLabel->setText(i18nc("@label", "%1 files selected", fileCount));
+		d->mMultipleFilesLabel->setText(i18ncp("@label", "%1 file selected", "%1 files selected", fileCount));
 	} else if (fileCount == 0) {
-		d->mMultipleFilesLabel->setText(i18ncp("@label", "One folder selected", "%1 folders selected", folderCount));
+		d->mMultipleFilesLabel->setText(i18ncp("@label", "%1 folder selected", "%1 folders selected", folderCount));
 	} else {
-		d->mMultipleFilesLabel->setText(i18nc("@label", "%1 folders and %2 files selected", folderCount, fileCount));
+		d->mMultipleFilesLabel->setText(i18nc("@label. The two parameters are strings like '2 folders' and '1 file'.",
+			"%1 and %2 selected", i18np("%1 folder", "%1 folders", folderCount), i18np("%1 file", "%1 files", fileCount)));
 	}
 	d->mOneFileWidget->hide();
 	d->mMultipleFilesLabel->show();
@@ -227,11 +224,6 @@ void InfoContextManagerItem::fillMultipleItemsGroup(const KFileItemList& itemLis
 
 
 void InfoContextManagerItem::updateOneFileInfo() {
-	if (!d->mSideBar) {
-		// Not initialized yet
-		return;
-	}
-
 	if (!d->mDocument) {
 		return;
 	}
@@ -260,7 +252,7 @@ void InfoContextManagerItem::showMetaInfoDialog() {
 		connect(d->mImageMetaInfoDialog, SIGNAL(preferredMetaInfoKeyListChanged(const QStringList&)),
 			SLOT(slotPreferredMetaInfoKeyListChanged(const QStringList&)) );
 	}
-	d->mImageMetaInfoDialog->setMetaInfo(d->mDocument->metaInfo(), GwenviewConfig::preferredMetaInfoKeyList());
+	d->mImageMetaInfoDialog->setMetaInfo(d->mDocument ? d->mDocument->metaInfo() : 0, GwenviewConfig::preferredMetaInfoKeyList());
 	d->mImageMetaInfoDialog->show();
 }
 
