@@ -1,6 +1,6 @@
 /*
 Gwenview: an image viewer
-Copyright 2007 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2007 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -44,11 +44,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <kmenubar.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include <knotificationrestrictions.h>
 #include <kprotocolmanager.h>
 #include <kstatusbar.h>
 #include <kstandardguiitem.h>
 #include <kstandardshortcut.h>
 #include <kactioncategory.h>
+#include <kprotocolmanager.h>
 #include <ktogglefullscreenaction.h>
 #include <ktoolbar.h>
 #include <kurl.h>
@@ -61,6 +63,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "contextmanager.h"
 #include "documentpanel.h"
 #include "fileopscontextmanageritem.h"
+#include "folderviewcontextmanageritem.h"
 #include "fullscreencontent.h"
 #include "gvcore.h"
 #include "imageopscontextmanageritem.h"
@@ -74,13 +77,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "preloader.h"
 #include "savebar.h"
 #include "sidebar.h"
+#include "splitter.h"
 #include "startpage.h"
-#include "thumbnailbarview.h"
 #include "thumbnailviewhelper.h"
 #include "thumbnailviewpanel.h"
 #include <lib/archiveutils.h>
 #include <lib/document/documentfactory.h>
 #include <lib/documentview/documentview.h>
+#include <lib/eventwatcher.h>
 #include <lib/fullscreenbar.h>
 #include <lib/gwenviewconfig.h>
 #include <lib/mimetypeutils.h>
@@ -88,6 +92,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <lib/slideshow.h>
 #include <lib/signalblocker.h>
 #include <lib/semanticinfo/sorteddirmodel.h>
+#include <lib/splittercollapser.h>
+#include <lib/thumbnailview/thumbnailbarview.h>
 #include <lib/thumbnailview/thumbnailview.h>
 #include <lib/thumbnailloadjob.h>
 #include <lib/urlutils.h>
@@ -105,6 +111,12 @@ namespace Gwenview {
 
 static const int PRELOAD_DELAY = 1000;
 
+static const char* BROWSE_MODE_SIDE_BAR_GROUP = "SideBar-BrowseMode";
+static const char* VIEW_MODE_SIDE_BAR_GROUP = "SideBar-ViewMode";
+static const char* FULLSCREEN_MODE_SIDE_BAR_GROUP = "SideBar-FullScreenMode";
+static const char* SIDE_BAR_IS_VISIBLE_KEY = "IsVisible";
+static const char* SIDE_BAR_CURRENT_PAGE_KEY = "CurrentPage";
+
 enum PageId {
 	StartPageId,
 	BrowsePageId,
@@ -112,8 +124,7 @@ enum PageId {
 };
 
 struct MainWindowState {
-	QAction* mActiveViewModeAction;
-	bool mSideBarVisible;
+	PageId mPageId;
 	bool mToolBarVisible;
 	Qt::WindowStates mWindowState;
 };
@@ -130,9 +141,8 @@ struct MainWindow::Private {
 	ThumbnailViewPanel* mThumbnailViewPanel;
 	StartPage* mStartPage;
 	SideBar* mSideBar;
+	SplitterCollapser* mSideBarCollapser;
 	QStackedWidget* mViewStackedWidget;
-	QStackedWidget* mSideBarContainer;
-	bool mSideBarWasVisibleBeforeTemporarySideBar;
 	FullScreenBar* mFullScreenBar;
 	FullScreenContent* mFullScreenContent;
 	SaveBar* mSaveBar;
@@ -149,7 +159,7 @@ struct MainWindow::Private {
 	KAction* mGoUpAction;
 	KAction* mGoToPreviousAction;
 	KAction* mGoToNextAction;
-	KAction* mToggleSideBarAction;
+	KToggleAction* mToggleSideBarAction;
 	KToggleFullScreenAction* mFullScreenAction;
 	KAction* mToggleSlideShowAction;
 	KToggleAction* mShowMenuBarAction;
@@ -157,6 +167,7 @@ struct MainWindow::Private {
 	SortedDirModel* mDirModel;
 	ContextManager* mContextManager;
 
+	bool mDistractionFreeMode;
 	MainWindowState mStateBeforeFullScreen;
 
 	KUrl mUrlToSelect;
@@ -166,21 +177,31 @@ struct MainWindow::Private {
 	PageId mCurrentPageId;
 
 	QDateTime mFullScreenLeftAt;
+	KNotificationRestrictions* mNotificationRestrictions;
 
 	void setupWidgets() {
 		QWidget* centralWidget = new QWidget(mWindow);
 		mWindow->setCentralWidget(centralWidget);
 		mSaveBar = new SaveBar(centralWidget, mWindow->actionCollection());
 
-		mCentralSplitter = new QSplitter(Qt::Horizontal, centralWidget);
+		mCentralSplitter = new Splitter(Qt::Horizontal, centralWidget);
 		QVBoxLayout* layout = new QVBoxLayout(centralWidget);
 		layout->addWidget(mSaveBar);
 		layout->addWidget(mCentralSplitter);
 		layout->setMargin(0);
 		layout->setSpacing(0);
 
+		mSideBar = new SideBar(mCentralSplitter);
+		EventWatcher::install(mSideBar, QList<QEvent::Type>() << QEvent::Show << QEvent::Hide,
+			mWindow, SLOT(updateToggleSideBarAction()));
+
 		mViewStackedWidget = new QStackedWidget(mCentralSplitter);
 
+		mSideBarCollapser = new SplitterCollapser(mCentralSplitter, mSideBar);
+
+		mStartSlideShowWhenDirListerCompleted = false;
+		mSlideShow = new SlideShow(mWindow);
+
 		setupThumbnailView(mViewStackedWidget);
 		setupDocumentPanel(mViewStackedWidget);
 		setupStartPage(mViewStackedWidget);
@@ -189,15 +210,8 @@ struct MainWindow::Private {
 		mViewStackedWidget->addWidget(mStartPage);
 		mViewStackedWidget->setCurrentWidget(mThumbnailViewPanel);
 
-		mSideBarContainer = new QStackedWidget(mCentralSplitter);
-		mSideBar = new SideBar(mSideBarContainer);
-		mSideBarContainer->addWidget(mSideBar);
-
-		mCentralSplitter->setStretchFactor(0, 1);
-		mCentralSplitter->setStretchFactor(1, 0);
-
-		mStartSlideShowWhenDirListerCompleted = false;
-		mSlideShow = new SlideShow(mWindow);
+		mCentralSplitter->setStretchFactor(0, 0);
+		mCentralSplitter->setStretchFactor(1, 1);
 
 		connect(mSaveBar, SIGNAL(requestSaveAll()),
 			mGvCore, SLOT(saveAll()) );
@@ -213,6 +227,10 @@ struct MainWindow::Private {
 
 		mThumbnailView = mThumbnailViewPanel->thumbnailView();
 		mUrlNavigator = mThumbnailViewPanel->urlNavigator();
+		QPalette pal = mUrlNavigator->palette();
+		pal.setColor(QPalette::Window, pal.color(QPalette::Window).dark(110));
+		mUrlNavigator->setAutoFillBackground(true);
+		mUrlNavigator->setPalette(pal);
 
 		mThumbnailViewHelper = new ThumbnailViewHelper(mDirModel, mWindow->actionCollection());
 		mThumbnailView->setThumbnailViewHelper(mThumbnailViewHelper);
@@ -242,7 +260,7 @@ struct MainWindow::Private {
 	}
 
 	void setupDocumentPanel(QWidget* parent) {
-		mDocumentPanel = new DocumentPanel(parent, mWindow->actionCollection());
+		mDocumentPanel = new DocumentPanel(parent, mSlideShow, mWindow->actionCollection());
 		connect(mDocumentPanel, SIGNAL(captionUpdateRequested(const QString&)),
 			mWindow, SLOT(setCaption(const QString&)) );
 		connect(mDocumentPanel, SIGNAL(completed()),
@@ -259,7 +277,7 @@ struct MainWindow::Private {
 	}
 
 	void setupStartPage(QWidget* parent) {
-		mStartPage = new StartPage(parent, mDirModel->semanticInfoBackEnd());
+		mStartPage = new StartPage(parent, mGvCore);
 		connect(mStartPage, SIGNAL(urlSelected(const KUrl&)),
 			mWindow, SLOT(slotStartPageUrlSelected(const KUrl&)) );
 	}
@@ -303,11 +321,10 @@ struct MainWindow::Private {
 		connect(mDocumentPanel, SIGNAL(toggleFullScreenRequested()),
 			mFullScreenAction, SLOT(trigger()) );
 
-
-		QShortcut* reduceLevelOfDetailsShortcut = new QShortcut(mWindow);
-		reduceLevelOfDetailsShortcut->setKey(Qt::Key_Escape);
-		connect(reduceLevelOfDetailsShortcut, SIGNAL(activated()),
-			mWindow, SLOT(reduceLevelOfDetails()) );
+		KAction* reduceLodAction = view->addAction("reduce_lod", mWindow, SLOT(reduceLevelOfDetails()));
+		// FIXME Find a better text for this action
+		reduceLodAction->setText(i18nc("@action Go back to a more general page (start page <- list <- image)", "Back"));
+		reduceLodAction->setShortcut(Qt::Key_Escape);
 
 		mGoToPreviousAction = view->addAction("go_previous",mWindow, SLOT(goToPrevious()));
 		mGoToPreviousAction->setIcon(KIcon("media-skip-backward"));
@@ -328,9 +345,12 @@ struct MainWindow::Private {
 		action->setIcon(KIcon("go-home"));
 		action->setText(i18nc("@action", "Start Page"));
 
-		mToggleSideBarAction = view->addAction("toggle_sidebar",mWindow, SLOT(toggleSideBar()));
+		mToggleSideBarAction = view->add<KToggleAction>("toggle_sidebar");
+		connect(mToggleSideBarAction, SIGNAL(toggled(bool)),
+			mWindow, SLOT(toggleSideBar(bool)));
 		mToggleSideBarAction->setIcon(KIcon("view-sidetree"));
 		mToggleSideBarAction->setShortcut(Qt::Key_F11);
+		mToggleSideBarAction->setText(i18nc("@action", "Sidebar"));
 
 		mToggleSlideShowAction = view->addAction("toggle_slideshow",mWindow, SLOT(toggleSlideShow()));
 		mWindow->updateSlideShowAction();
@@ -377,29 +397,58 @@ struct MainWindow::Private {
 		KActionCollection* actionCollection = mWindow->actionCollection();
 
 		mContextManager = new ContextManager(mWindow);
-		mContextManager->setSideBar(mSideBar);
 		mContextManager->setDirModel(mDirModel);
 
-		mContextManager->addItem(new InfoContextManagerItem(mContextManager));
+		// Create context manager items
+		FolderViewContextManagerItem* folderViewItem = new FolderViewContextManagerItem(mContextManager);
+		mContextManager->addItem(folderViewItem);
+		connect(folderViewItem, SIGNAL(urlChanged(const KUrl&)),
+			mWindow, SLOT(openDirUrl(const KUrl&)));
+
+		InfoContextManagerItem* infoItem = new InfoContextManagerItem(mContextManager);
+		mContextManager->addItem(infoItem);
 
 		#ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
-		mContextManager->addItem(new SemanticInfoContextManagerItem(mContextManager, actionCollection, mDocumentPanel->documentView()));
+		SemanticInfoContextManagerItem* semanticInfoItem =
+			new SemanticInfoContextManagerItem(mContextManager, actionCollection, mDocumentPanel->documentView());
+		mContextManager->addItem(semanticInfoItem);
 		#endif
 
-		mContextManager->addItem(new ImageOpsContextManagerItem(mContextManager, mWindow));
+		ImageOpsContextManagerItem* imageOpsItem =
+			new ImageOpsContextManagerItem(mContextManager, mWindow);
+		mContextManager->addItem(imageOpsItem);
 
 		FileOpsContextManagerItem* fileOpsItem = new FileOpsContextManagerItem(mContextManager, actionCollection);
 		mContextManager->addItem(fileOpsItem);
+
+		// Fill sidebar
+		SideBarPage* page;
+		page = new SideBarPage(i18n("Folders"));
+		page->setObjectName("folders");
+		page->addWidget(folderViewItem->widget());
+		page->layout()->setMargin(0);
+		mSideBar->addPage(page);
+
+		page = new SideBarPage(i18n("Information"));
+		page->setObjectName("information");
+		page->addWidget(infoItem->widget());
+		#ifndef GWENVIEW_SEMANTICINFO_BACKEND_NONE
+		page->addWidget(semanticInfoItem->widget());
+		#endif
+		page->addStretch();
+		mSideBar->addPage(page);
+
+		page = new SideBarPage(i18n("Operations"));
+		page->setObjectName("operations");
+		page->addWidget(imageOpsItem->widget());
+		page->addWidget(fileOpsItem->widget());
+		page->addStretch();
+		mSideBar->addPage(page);
 	}
 
 
 	void initDirModel() {
-		KDirLister* dirLister = mDirModel->dirLister();
-		QStringList mimeTypes;
-		mimeTypes += MimeTypeUtils::dirMimeTypes();
-		mimeTypes += MimeTypeUtils::imageMimeTypes();
-		mimeTypes += MimeTypeUtils::videoMimeTypes();
-		dirLister->setMimeFilter(mimeTypes);
+		setDirModelShowDirs(true);
 
 		connect(mDirModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)),
 			mWindow, SLOT(slotDirModelNewItems()) );
@@ -409,16 +458,19 @@ struct MainWindow::Private {
 		connect(mDirModel, SIGNAL(modelReset()),
 			mWindow, SLOT(updatePreviousNextActions()) );
 
-		connect(dirLister, SIGNAL(completed()),
+		connect(mDirModel->dirLister(), SIGNAL(completed()),
 			mWindow, SLOT(slotDirListerCompleted()) );
 	}
 
-	void updateToggleSideBarAction() {
-		if (mSideBarContainer->isVisibleTo(mSideBarContainer->parentWidget())) {
-			mToggleSideBarAction->setText(i18n("Hide Sidebar"));
-		} else {
-			mToggleSideBarAction->setText(i18n("Show Sidebar"));
+	void setDirModelShowDirs(bool showDirs) {
+		MimeTypeUtils::Kinds kinds =
+			MimeTypeUtils::KIND_RASTER_IMAGE
+			| MimeTypeUtils::KIND_SVG_IMAGE
+			| MimeTypeUtils::KIND_VIDEO;
+		if (showDirs) {
+			kinds |= MimeTypeUtils::KIND_DIR | MimeTypeUtils::KIND_ARCHIVE;
 		}
+		mDirModel->setKindFilter(kinds);
 	}
 
 	QModelIndex getRelativeIndex(int offset) {
@@ -514,6 +566,11 @@ struct MainWindow::Private {
 		setActionEnabled("add_folder_to_places", enabled);
 	}
 
+	void updateDistractionsState() {
+		const bool isFullScreen = mFullScreenAction->isChecked();
+		mSideBarCollapser->setVisible(!mDistractionFreeMode && !isFullScreen);
+		mFullScreenBar->setActivated(!mDistractionFreeMode && isFullScreen);
+	}
 
 	void updateActions() {
 		bool isRasterImage = false;
@@ -602,6 +659,59 @@ struct MainWindow::Private {
 		mSaveBar->setCurrentUrl(url);
 		mSlideShow->setCurrentUrl(url);
 	}
+
+	const char* sideBarConfigGroupName() const {
+		const char* name = 0;
+		switch (mCurrentPageId) {
+		case StartPageId:
+			kWarning() << "Should not happen!";
+			// Fall through
+		case BrowsePageId:
+			name = BROWSE_MODE_SIDE_BAR_GROUP;
+			break;
+		case ViewPageId:
+			name = mDocumentPanel->isFullScreenMode()
+				? FULLSCREEN_MODE_SIDE_BAR_GROUP
+				: VIEW_MODE_SIDE_BAR_GROUP;
+			break;
+		}
+		return name;
+	}
+
+	void loadSideBarConfig() {
+		static QMap<const char*, bool> defaultVisibility;
+		static QMap<const char*, QString> defaultPage;
+		if (defaultVisibility.isEmpty()) {
+			defaultVisibility[BROWSE_MODE_SIDE_BAR_GROUP]     = true;
+			defaultVisibility[VIEW_MODE_SIDE_BAR_GROUP]       = true;
+			defaultVisibility[FULLSCREEN_MODE_SIDE_BAR_GROUP] = false;
+
+			defaultPage[BROWSE_MODE_SIDE_BAR_GROUP]           = "folders";
+			defaultPage[VIEW_MODE_SIDE_BAR_GROUP]             = "information";
+			defaultPage[FULLSCREEN_MODE_SIDE_BAR_GROUP]       = "information";
+		}
+
+		const char* name = sideBarConfigGroupName();
+		KConfigGroup group(KGlobal::config(), name);
+		mSideBar->setVisible(group.readEntry(SIDE_BAR_IS_VISIBLE_KEY, defaultVisibility[name]));
+		mSideBar->setCurrentPage(group.readEntry(SIDE_BAR_CURRENT_PAGE_KEY, defaultPage[name]));
+	}
+
+	void saveSideBarConfig() const {
+		KConfigGroup group(KGlobal::config(), sideBarConfigGroupName());
+		group.writeEntry(SIDE_BAR_IS_VISIBLE_KEY, mSideBar->isVisible());
+		group.writeEntry(SIDE_BAR_CURRENT_PAGE_KEY, mSideBar->currentPage());
+	}
+
+	void setScreenSaverEnabled(bool enabled) {
+		// Always delete mNotificationRestrictions, it does not hurt
+		delete mNotificationRestrictions;
+		if (!enabled) {
+			mNotificationRestrictions = new KNotificationRestrictions(KNotificationRestrictions::ScreenSaver, mWindow);
+		} else {
+			mNotificationRestrictions = 0;
+		}
+	}
 };
 
 
@@ -611,9 +721,11 @@ d(new MainWindow::Private)
 {
 	d->mWindow = this;
 	d->mCurrentPageId = StartPageId;
+	d->mDistractionFreeMode = false;
 	d->mDirModel = new SortedDirModel(this);
 	d->mGvCore = new GvCore(this, d->mDirModel);
 	d->mPreloader = new Preloader(this);
+	d->mNotificationRestrictions = 0;
 	d->initDirModel();
 	d->setupWidgets();
 	d->setupActions();
@@ -623,7 +735,6 @@ d(new MainWindow::Private)
 	d->updateActions();
 	updatePreviousNextActions();
 	d->mSaveBar->initActionDependentWidgets();
-	d->mThumbnailViewPanel->initActionDependentWidgets();
 
 	createGUI();
 	loadConfig();
@@ -672,25 +783,6 @@ bool MainWindow::currentDocumentIsRasterImage() const {
 }
 
 
-void MainWindow::showTemporarySideBar(QWidget* sideBar) {
-	d->mSideBarWasVisibleBeforeTemporarySideBar = d->mSideBarContainer->isVisible();
-	// Move the sideBar inside a widget so that we can add a stretch below
-	// it.
-	QWidget* widget = new QWidget(d->mSideBarContainer);
-	sideBar->setParent(widget);
-
-	QVBoxLayout* layout = new QVBoxLayout(widget);
-	layout->setMargin(0);
-	layout->setSpacing(0);
-	layout->addWidget(sideBar);
-	layout->addStretch();
-
-	d->mSideBarContainer->addWidget(widget);
-	d->mSideBarContainer->setCurrentWidget(widget);
-	d->mSideBarContainer->show();
-}
-
-
 void MainWindow::setCaption(const QString& caption) {
 	// Keep a trace of caption to use it in slotModifiedDocumentListChanged()
 	d->mCaption = caption;
@@ -713,8 +805,12 @@ void MainWindow::slotModifiedDocumentListChanged() {
 }
 
 
-void MainWindow::setInitialUrl(const KUrl& url) {
-	Q_ASSERT(url.isValid());
+void MainWindow::setInitialUrl(const KUrl& _url) {
+	Q_ASSERT(_url.isValid());
+	KUrl url = UrlUtils::fixUserEnteredUrl(_url);
+	if (url.protocol() == "http" || url.protocol() == "https") {
+		d->mGvCore->addUrlToRecentUrls(url);
+	}
 	if (UrlUtils::urlIsDirectory(url)) {
 		d->mBrowseAction->trigger();
 		openDirUrl(url);
@@ -723,8 +819,6 @@ void MainWindow::setInitialUrl(const KUrl& url) {
 		openDocumentUrl(url);
 	}
 	d->updateContextDependentComponents();
-	d->mSideBarContainer->setVisible(GwenviewConfig::sideBarIsVisible());
-	d->updateToggleSideBarAction();
 }
 
 
@@ -737,11 +831,15 @@ void MainWindow::startSlideShow() {
 
 
 void MainWindow::setActiveViewModeAction(QAction* action) {
+	if (d->mCurrentPageId == StartPageId) {
+		d->mSideBarCollapser->show();
+	} else {
+		d->saveSideBarConfig();
+	}
 	if (action == d->mViewAction) {
 		d->mCurrentPageId = ViewPageId;
 		// Switching to view mode
-		QStringList mimeFilter = MimeTypeUtils::dirMimeTypes() + ArchiveUtils::mimeTypes();
-		d->mDirModel->setMimeExcludeFilter(mimeFilter);
+		d->setDirModelShowDirs(false);
 		d->mViewStackedWidget->setCurrentWidget(d->mDocumentPanel);
 		if (d->mDocumentPanel->isEmpty()) {
 			openSelectedDocument();
@@ -760,9 +858,10 @@ void MainWindow::setActiveViewModeAction(QAction* action) {
 			// his image back.
 			d->mDocumentPanel->reset();
 		}
-		d->mDirModel->setMimeExcludeFilter(QStringList());
+		d->setDirModelShowDirs(true);
 		setCaption(QString());
 	}
+	d->loadSideBarConfig();
 
 	emit viewModeChanged();
 }
@@ -777,15 +876,17 @@ void MainWindow::slotThumbnailViewIndexActivated(const QModelIndex& index) {
 	if (item.isDir()) {
 		// Item is a dir, open it
 		openDirUrl(item.url());
-	} else if (ArchiveUtils::fileItemIsArchive(item)) {
-		// Item is an archive, tweak url then open it
-		KUrl url = item.url();
-		QString protocol = ArchiveUtils::protocolForMimeType(item.mimetype());
-		url.setProtocol(protocol);
-		openDirUrl(url);
 	} else {
-		// Item is a document, switch to view mode
-		d->mViewAction->trigger();
+		QString protocol = ArchiveUtils::protocolForMimeType(item.mimetype());
+		if (!protocol.isEmpty()) {
+			// Item is an archive, tweak url then open it
+			KUrl url = item.url();
+			url.setProtocol(protocol);
+			openDirUrl(url);
+		} else {
+			// Item is a document, switch to view mode
+			d->mViewAction->trigger();
+		}
 	}
 }
 
@@ -808,19 +909,26 @@ void MainWindow::openSelectedDocument() {
 
 
 void MainWindow::goUp() {
-	KUrl url = d->mDirModel->dirLister()->url();
-	url = url.upUrl();
-	openDirUrl(url);
+	if (d->mCurrentPageId == BrowsePageId) {
+		KUrl url = d->mDirModel->dirLister()->url();
+		url = url.upUrl();
+		openDirUrl(url);
+	} else {
+		d->mBrowseAction->trigger();
+	}
 }
 
 
 void MainWindow::showStartPage() {
-	d->mCurrentPageId = StartPageId;
+	if (d->mCurrentPageId != StartPageId) {
+		d->saveSideBarConfig();
+		d->mCurrentPageId = StartPageId;
+	}
 	d->setActionsDisabledOnStartPageEnabled(false);
 	d->spreadCurrentDirUrl(KUrl());
 
-	d->mSideBarContainer->hide();
-	d->updateToggleSideBarAction();
+	d->mSideBar->hide();
+	d->mSideBarCollapser->hide();
 	d->mViewStackedWidget->setCurrentWidget(d->mStartPage);
 
 	d->updateActions();
@@ -886,9 +994,14 @@ void MainWindow::openDocumentUrl(const KUrl& url) {
 }
 
 
-void MainWindow::toggleSideBar() {
-	d->mSideBarContainer->setVisible(!d->mSideBarContainer->isVisible());
-	d->updateToggleSideBarAction();
+void MainWindow::toggleSideBar(bool on) {
+	d->mSideBar->setVisible(on);
+}
+
+
+void MainWindow::updateToggleSideBarAction() {
+	SignalBlocker blocker(d->mToggleSideBarAction);
+	d->mToggleSideBarAction->setChecked(d->mSideBar->isVisible());
 }
 
 
@@ -938,7 +1051,6 @@ void MainWindow::slotSelectionChanged() {
 	}
 
 	// Update UI
-	hideTemporarySideBar();
 	d->updateActions();
 	updatePreviousNextActions();
 	d->updateContextDependentComponents();
@@ -1019,49 +1131,64 @@ void MainWindow::reduceLevelOfDetails() {
 		d->mFullScreenAction->trigger();
 	} else if (d->mCurrentPageId == ViewPageId) {
 		d->mBrowseAction->trigger();
+	} else {
+		showStartPage();
 	}
 }
 
 
 void MainWindow::toggleFullScreen(bool checked) {
 	setUpdatesEnabled(false);
+	d->saveSideBarConfig();
 	if (checked) {
 		// Save MainWindow config now, this way if we quit while in
 		// fullscreen, we are sure latest MainWindow changes are remembered.
 		saveMainWindowSettings(autoSaveConfigGroup());
 		resetAutoSaveSettings();
 
-		// Go full screen
-		d->mStateBeforeFullScreen.mActiveViewModeAction = d->mViewModeActionGroup->checkedAction();
-		d->mStateBeforeFullScreen.mSideBarVisible = d->mSideBarContainer->isVisible();
+		// Save state
+		d->mStateBeforeFullScreen.mPageId = d->mCurrentPageId;
 		d->mStateBeforeFullScreen.mToolBarVisible = toolBar()->isVisible();
 		d->mStateBeforeFullScreen.mWindowState = windowState();
 
+		// Go full screen
 		d->mViewAction->trigger();
-		d->mSideBarContainer->hide();
 
 		setWindowState(windowState() | Qt::WindowFullScreen);
 		menuBar()->hide();
 		toolBar()->hide();
 		d->mDocumentPanel->setFullScreenMode(true);
 		d->mSaveBar->setFullScreenMode(true);
-		d->mFullScreenBar->setActivated(true);
+		d->updateDistractionsState();
+		d->setScreenSaverEnabled(false);
+
+		// HACK: Only load sidebar config now, because it looks at
+		// DocumentPanel fullScreenMode property to determine the sidebar
+		// config group.
+		d->loadSideBarConfig();
 	} else {
 		setAutoSaveSettings();
-		if (d->mStateBeforeFullScreen.mActiveViewModeAction) {
-			d->mStateBeforeFullScreen.mActiveViewModeAction->trigger();
-		}
-		d->mSideBarContainer->setVisible(d->mStateBeforeFullScreen.mSideBarVisible);
 
 		// Back to normal
+		d->mCurrentPageId = d->mStateBeforeFullScreen.mPageId;
+		if (d->mCurrentPageId == BrowsePageId) {
+			d->mBrowseAction->trigger();
+		}
+
 		d->mDocumentPanel->setFullScreenMode(false);
 		d->mSlideShow->stop();
 		d->mSaveBar->setFullScreenMode(false);
-		d->mFullScreenBar->setActivated(false);
 		setWindowState(d->mStateBeforeFullScreen.mWindowState);
 		menuBar()->setVisible(d->mShowMenuBarAction->isChecked());
 		toolBar()->setVisible(d->mStateBeforeFullScreen.mToolBarVisible);
 
+		d->updateDistractionsState();
+		d->setScreenSaverEnabled(true);
+
+		// Keep this after mDocumentPanel->setFullScreenMode(false).
+		// See call to loadSideBarConfig() above.
+		d->loadSideBarConfig();
+
 		// See resizeEvent
 		d->mFullScreenLeftAt = QDateTime::currentDateTime();
 	}
@@ -1093,7 +1220,7 @@ void MainWindow::openFile() {
 
 	KFileDialog dialog(dirUrl, QString(), this);
 	dialog.setCaption(i18nc("@title:window", "Open Image"));
-	QStringList mimeFilter = MimeTypeUtils::imageMimeTypes();
+	const QStringList mimeFilter = MimeTypeUtils::imageMimeTypes();
 	dialog.setMimeFilter(mimeFilter);
 	dialog.setOperationMode(KFileDialog::Opening);
 	if (!dialog.exec()) {
@@ -1123,8 +1250,14 @@ void MainWindow::toggleSlideShow() {
 			QModelIndex index = d->mDirModel->index(pos, 0);
 			KFileItem item = d->mDirModel->itemForIndex(index);
 			MimeTypeUtils::Kind kind = MimeTypeUtils::fileItemKind(item);
-			if (kind == MimeTypeUtils::KIND_FILE || kind == MimeTypeUtils::KIND_RASTER_IMAGE) {
+			switch (kind) {
+			case MimeTypeUtils::KIND_SVG_IMAGE:
+			case MimeTypeUtils::KIND_RASTER_IMAGE:
+			case MimeTypeUtils::KIND_VIDEO:
 				list << item.url();
+				break;
+			default:
+				break;
 			}
 		}
 		d->mSlideShow->start(list);
@@ -1156,6 +1289,7 @@ void MainWindow::generateThumbnailForUrl(const KUrl& url) {
 
 bool MainWindow::queryClose() {
 	saveConfig();
+	d->saveSideBarConfig();
 	QList<KUrl> list = DocumentFactory::instance()->modifiedDocumentList();
 	if (list.size() == 0) {
 		return true;
@@ -1213,6 +1347,12 @@ void MainWindow::toggleMenuBar() {
 
 
 void MainWindow::loadConfig() {
+	d->mDirModel->setBlackListedExtensions(GwenviewConfig::blackListedExtensions());
+
+	d->mDocumentPanel->loadConfig();
+	d->mThumbnailViewPanel->loadConfig();
+
+	// Colors
 	QColor bgColor = GwenviewConfig::viewBackgroundColor();
 	QColor fgColor = bgColor.value() > 128 ? Qt::black : Qt::white;
 
@@ -1220,21 +1360,24 @@ void MainWindow::loadConfig() {
 	pal.setColor(QPalette::Base, bgColor);
 	pal.setColor(QPalette::Text, fgColor);
 
+	// Apply to widgets
 	d->mThumbnailViewPanel->applyPalette(pal);
 	d->mStartPage->applyPalette(pal);
 	d->mDocumentPanel->setNormalPalette(pal);
 
-	d->mDirModel->setBlackListedExtensions(GwenviewConfig::blackListedExtensions());
-
-	d->mDocumentPanel->loadConfig();
-	d->mThumbnailViewPanel->loadConfig();
+	// FIXME: Should we avoid CSS here to get a more native look?
+	d->mSideBarCollapser->setAutoFillBackground(true);
+	d->mSideBarCollapser->setStyleSheet(
+		"	background-color: palette(window);"
+		"	border: 1px solid palette(mid);"
+		"	border-radius: 5px;"
+		);
 }
 
 
 void MainWindow::saveConfig() {
 	d->mDocumentPanel->saveConfig();
 	d->mThumbnailViewPanel->saveConfig();
-	GwenviewConfig::setSideBarIsVisible(d->mSideBarContainer->isVisible());
 }
 
 
@@ -1249,20 +1392,6 @@ void MainWindow::print() {
 }
 
 
-void MainWindow::hideTemporarySideBar() {
-	QWidget* temporarySideBar = d->mSideBarContainer->currentWidget();
-	if (temporarySideBar == d->mSideBar) {
-		return;
-	}
-	temporarySideBar->deleteLater();
-
-	if (!d->mSideBarWasVisibleBeforeTemporarySideBar) {
-		d->mSideBarContainer->hide();
-	}
-	d->mSideBarContainer->setCurrentWidget(d->mSideBar);
-}
-
-
 void MainWindow::preloadNextUrl() {
 	QItemSelection selection = d->mThumbnailView->selectionModel()->selection();
 	if (selection.size() != 1) {
@@ -1320,4 +1449,10 @@ void MainWindow::resizeEvent(QResizeEvent* event) {
 }
 
 
+void MainWindow::setDistractionFreeMode(bool value) {
+	d->mDistractionFreeMode = value;
+	d->updateDistractionsState();
+}
+
+
 } // namespace
