@@ -1,6 +1,6 @@
 /*
 Gwenview: an image viewer
-Copyright 2007-2008 Aurélien Gâteau <aurelien.gateau@free.fr>
+Copyright 2007-2008 Aurélien Gâteau <agateau@kde.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -55,7 +55,7 @@ namespace Gwenview {
 GVPart::GVPart(QWidget* parentWidget, QObject* parent, const QStringList& /*args*/)
 : KParts::ReadOnlyPart(parent)
 {
-	mDocumentView = new DocumentView(parentWidget, actionCollection());
+	mDocumentView = new DocumentView(parentWidget, 0 /* slideShow */, actionCollection());
 	mDocumentView->setZoomWidgetVisible(false);
 	setWidget(mDocumentView);
 
@@ -123,7 +123,7 @@ KAboutData* GVPart::createAboutData() {
 	aboutData->addAuthor(
 		ki18n("Aurélien Gâteau"),
 		ki18n("Main developer"),
-		"aurelien.gateau@free.fr");
+		"agateau@kde.org");
 	return aboutData;
 }
 
