@@ -50,7 +50,6 @@
 #include <kfiledialog.h>
 #include <kglobal.h>
 #include <kiconloader.h>
-#include <kimagefilepreview.h>
 #include <kimageio.h>
 #include <kio/netaccess.h>
 #include <klocale.h>
@@ -62,6 +61,7 @@
 #include <ktoolinvocation.h>
 #include <kdeprintdialog.h>
 #include <kprintpreview.h>
+#include <kurlcombobox.h>
 
 #include <kpCommandHistory.h>
 #include <kpDefs.h>
@@ -439,9 +439,6 @@ KUrl::List kpMainWindow::askForOpenURLs (const QString &caption, const QString &
     if (allowMultipleURLs)
         fd.setMode (KFile::Files);
 
-    // COMPAT: Doesn't work.  KFileDialog bug.
-    fd.setPreviewWidget (new KImageFilePreview (&fd));
-
     if (fd.exec ())
         return fd.selectedUrls ();
     else
@@ -829,6 +826,10 @@ KUrl kpMainWindow::askForSaveURL (const QString &caption,
     connect (&fd, SIGNAL (filterChanged (const QString &)),
              saveOptionsWidget, SLOT (setMimeType (const QString &)));
 
+    //Honor the current filename if the mimetype is the same
+    KMimeType::Ptr mime = KMimeType::findByUrl(startURL);
+    if (mime->is(fdSaveOptions.mimeType()))
+        fd.locationEdit()->setUrl(KUrl(startURL).fileName());
 
     if (fd.exec ())
     {
