@@ -48,62 +48,74 @@ class LIBKDCRAW_EXPORT KDcraw : public QObject
 
 public:
 
-    /** Standard constructor. */
+    /** Standard constructor.
+     */
     KDcraw();
 
-    /** Standard destructor. */
+    /** Standard destructor.
+     */
     virtual ~KDcraw();
 
 public:
 
-    /** Return a string version of libkdcraw release */
+    /** Return a string version of libkdcraw release
+     */
     static QString version();
 
     /** This is a non cancelable method witch do not require a class instance to run. 
         It can loadEmbeddedPreview() in first and if it failed, call loadHalfPreview(). 
-    */
+     */
     static bool loadDcrawPreview(QImage& image, const QString& path);
 
     /** Get the embedded JPEG preview image from RAW picture as a QByteArray witch will include Exif Data. 
         This is a fast and non cancelable. This method do not require a class instance to run.
-    */
+     */
     static bool loadEmbeddedPreview(QByteArray& imgData, const QString& path);
 
     /** Get the embedded JPEG preview image from RAW picture has a QImage. This is a fast and non cancelable 
         This method do not require a class instance to run.
-    */
+     */
     static bool loadEmbeddedPreview(QImage& image, const QString& path);
 
     /** Get the half decode RAW picture. This is a more slower than loadEmbeddedPreview() method
         and non cancelable. This method do not require a class instance to run.
-    */
+     */
     static bool loadHalfPreview(QImage& image, const QString& path);
 
     /** Get the camera settings witch have taken RAW file. Look into dcrawinfocontainer.h 
         for more details. This is a fast and non cancelable method witch do not require 
         a class instance to run.
-    */ 
+     */
     static bool rawFileIdentify(DcrawInfoContainer& identify, const QString& path);
 
-    /** Return the string of all RAW file type mime supported. */
+    /** Return the string of all RAW file type mime supported. 
+     */
     static const char *rawFiles();
 
     /** Return the list of all RAW file type mime supported,
-        as a QStringList, without wildcard and suffix dot.  */
+        as a QStringList, without wildcard and suffix dot.  
+     */
     static QStringList rawFilesList();
 
     /** Returns a version number for the list of supported RAW file types.
         This version is incremented if the list of supported formats has changed
-        between library releases. */
+        between library releases. 
+     */
     static int rawFilesVersion();
 
-    /** Provide a list of supported RAW Camera name. */ 
+    /** Provide a list of supported RAW Camera name.
+     */ 
     static QStringList supportedCamera();
 
-    /** Return LibRaw version string. */
+    /** Return LibRaw version string. 
+     */
     static QString librawVersion();
 
-public: 
+    /** Return true if LibRaw use parallel demosaicing (libgomp support).
+     */
+    static bool librawUseGomp();
+
+public:
 
     /** Extract Raw image data undemosaiced and without post processing from 'filePath' picture file. 
         This is a cancelable method which require a class instance to run because RAW pictures loading 
@@ -114,7 +126,7 @@ public:
             - A byte array container 'rawData' with raw data. 
             - All info about Raw image into 'identify' container. 
             - 'false' is returned if loadding failed, else 'true'.  
-    */
+     */
     bool extractRAWData(const QString& filePath, QByteArray &rawData, DcrawInfoContainer& identify);
 
     /** Extract a small size of decode RAW data from 'filePath' picture file using 
@@ -130,7 +142,7 @@ public:
             - Size size of image in number of pixels ('width' and 'height').
             - The max average of RGB components from decoded picture.
             - 'false' is returned if decoding failed, else 'true'.  
-    */
+     */
     bool decodeHalfRAWImage(const QString& filePath, const RawDecodingSettings& rawDecodingSettings, 
                             QByteArray &imageData, int &width, int &height, int &rgbmax);
 
@@ -147,25 +159,25 @@ public:
             - Size size of image in number of pixels ('width' and 'height').
             - The max average of RGB components from decoded picture. 
             - 'false' is returned if decoding failed, else 'true'.  
-    */
+     */
     bool decodeRAWImage(const QString& filePath, const RawDecodingSettings& rawDecodingSettings, 
                         QByteArray &imageData, int &width, int &height, int &rgbmax);
 
     /** To cancel 'decodeHalfRAWImage' and 'decodeRAWImage' methods running 
         in a separate thread.
-    */
+     */
     void cancel();
 
 protected:
 
     /** Used internally to cancel RAW decoding operation. Normally, you don't need to use it 
         directly, excepted if you derivated this class. Usual way is to use cancel() method 
-    */
+     */
     bool                m_cancel;
 
     /** The settings container used to perform RAW pictures decoding. See 'rawdecodingsetting.h' 
         for details.
-    */
+     */
     RawDecodingSettings m_rawDecodingSettings;
 
 protected:
@@ -173,21 +185,21 @@ protected:
     /** Re-implement this method to control the cancelisation of loop witch wait data 
         from RAW decoding process with your propers envirronement. 
         By default, this method check if m_cancel is true.
-    */
+     */
     virtual bool checkToCancelWaitingData();
 
     /** This method is obsolete and will be removed in the future.
-    */
+     */
     virtual KDE_DEPRECATED bool checkToCancelReceivingData();
 
     /** Re-implement this method to control the pseudo progress value during RAW decoding (when dcraw run with an
         internal loop without feedback) with your proper environment. By default, this method do nothing.
         Progress value average for this stage is 0%-n%, with 'n' == 40% max (see setWaitingDataProgress() method).
-    */
+     */
     virtual void setWaitingDataProgress(double value);
 
     /** This method is obsolete and will be removed in the future.
-    */
+     */
     virtual KDE_DEPRECATED void setReceivingDataProgress(double value);
 
 private:
@@ -197,7 +209,7 @@ private:
 
 private:
 
-    KDcrawPriv *d;
+    KDcrawPriv* const d;
 
     friend class KDcrawPriv;
 };
