@@ -6,14 +6,15 @@
  * Date        : 2006-09-15
  * Description : Exiv2 library interface for KDE
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * Exiv2: http://www.exiv2.org
  * Exif : http://www.exif.org/Exif2-2.PDF
  * Iptc : http://www.iptc.org/std/IIM/4.1/specification/IIMV4.1.pdf
  * Xmp  : http://www.adobe.com/devnet/xmp/pdfs/xmp_specification.pdf
  *        http://www.iptc.org/std/Iptc4xmpCore/1.0/specification/Iptc4xmpCore_1.0-spec-XMPSchema_8.pdf
+ * Paper: http://www.metadataworkinggroup.com/pdf/mwg_guidance.pdf
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -118,10 +119,28 @@ public:
     //-- STATICS methods ----------------------------------------------
     //-----------------------------------------------------------------
 
+    /** Return true if Exiv2 library initialization is done properlly.
+        This method must be call before to use multithreading with libkexiv2.
+        It initialize several non re-entrancy code from Adobe XMP SDK 
+        See B.K.O #166424 for details.
+     */
+    static bool initializeExiv2();
+
+    /** Return true if Exiv2 library memory allocations are cleaned properlly.
+        This method must be call after to use multithreading with libkexiv2.
+        It cleanup memory used by Adobe XMP SDK 
+        See B.K.O #166424 for details.
+     */
+    static bool cleanupExiv2();
+
     /** Return true if library can handle Xmp metadata
      */
     static bool supportXmp();
 
+    /** Return true if library can writte metadata to typeMime file format.
+     */
+    static bool supportMetadataWritting(const QString& typeMime);
+
     /** Return a string version of Exiv2 release in format "major.minor.patch"
      */
     static QString Exiv2Version();
@@ -131,13 +150,20 @@ public:
     static QString version();
 
     /** return true if metadata from file cannot be written by Exiv2.
+        This method is obosolete and will be removed.
+        Use canWriteComment(), canWriteExif(), canWriteIptc(), or canWriteXmp() instead.
      */
-    static bool isReadOnly(const QString& filePath);
+    KDE_DEPRECATED static bool isReadOnly(const QString& filePath);
 
     //-----------------------------------------------------------------
     //-- GENERAL methods ----------------------------------------------
     //-----------------------------------------------------------------
 
+    /** Load all metadata (Exif, Iptc, Xmp, and JFIF Comments) from a byte array. 
+        Return true if metadata have been loaded successfully from image data.
+     */
+    bool load(const QByteArray& imgData) const;
+
     /** Load all metadata (Exif, Iptc, Xmp, and JFIF Comments) from a picture (JPEG, RAW, TIFF, PNG,
         DNG, etc...). Return true if metadata have been loaded successfully from file.
      */
@@ -165,6 +191,25 @@ public:
      */
     QString getFilePath() const;
 
+    /** Enable or disable writing metadata operations to RAW tiff based files.
+        It's require Exiv2 0.18. By default RAW files are untouched.
+     */
+    void setWriteRawFiles(bool on);
+
+    /** Return true if writing metadata operations on RAW tiff based files is enabled.
+        It's require Exiv2 0.18.
+     */
+    bool writeRawFiles() const;
+
+    /** Enable or disable file timestamp updating when metadata are saved.
+        By default files timestamp are untouched.
+     */
+    void setUpdateFileTimeStamp(bool on);
+
+    /** Return true if file timestamp is updated when metadata are saved.
+     */
+    bool updateFileTimeStamp() const;
+
     //-------------------------------------------------------------------
     //-- Metadata IMAGE INFORMATION manipulation methods ----------------
     //-------------------------------------------------------------------
@@ -384,7 +429,7 @@ public:
                            bool rationalWantSmallDenominator=true, bool setProgramName=true) const;
 
     /** Remove the Exif tag 'exifTagName' from Exif metadata. Return true if tag is 
-        removed successfully.
+        removed successfully or if no tag was present.
      */
     bool removeExifTag(const char *exifTagName, bool setProgramName=true) const;
 
@@ -485,7 +530,7 @@ public:
     bool setIptcTagData(const char *iptcTagName, const QByteArray& data, bool setProgramName=true) const;
 
     /** Remove the all instance of Iptc tags 'iptcTagName' from Iptc metadata. Return true if all 
-        tags have been removed successfully.
+        tags have been removed successfully (or none were present).
      */
     bool removeIptcTag(const char *iptcTagName, bool setProgramName=true) const;
 
@@ -719,7 +764,7 @@ public:
     bool registerXmpNameSpace(const QString& uri, const QString& prefix) const;
 
     /** Remove the Xmp tag 'xmpTagName' from Xmp metadata. Return true if tag is 
-        removed successfully.
+        removed successfully or if no tag was present.
      */
     bool removeXmpTag(const char *xmpTagName, bool setProgramName=true) const;
 
@@ -846,7 +891,7 @@ private:
 
     /** Internal class to store private members. Used to improve binary compatibility
      */
-    KExiv2Priv *d;
+    KExiv2Priv* const d;
 };
 
 }  // NameSpace KExiv2Iface
