@@ -24,7 +24,7 @@
 
 // Local includes.
 
-#include "kexiv2private.h"
+#include "kexiv2_p.h"
 #include "kexiv2.h"
 
 namespace KExiv2Iface
@@ -231,32 +231,33 @@ bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
 
     try
     {
-        if (comment.isEmpty())
-            return false;
+        removeExifTag("Exif.Photo.UserComment");
 
-    // Write as Unicode only when necessary.
-    QTextCodec *latin1Codec = QTextCodec::codecForName("iso8859-1");
-    if (latin1Codec->canEncode(comment))
-    {
-        // write as ASCII
-        std::string exifComment("charset=\"Ascii\" ");
-        exifComment += comment.toLatin1().constData();
-        d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
-    }
-    else
-    {
-        // write as Unicode (UCS-2)
-
-        // Be aware that we are dealing with a UCS-2 string.
-        // Null termination means \0\0, strlen does not work,
-        // do not use any const-char*-only methods,
-        // pass a std::string and not a const char * to ExifDatum::operator=().
-        const unsigned short *ucs2 = comment.utf16();
-        std::string exifComment("charset=\"Unicode\" ");
-        exifComment.append((const char*)ucs2, sizeof(unsigned short) * comment.length());
-        d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
-    }
+        if (!comment.isNull())
+        {
+            // Write as Unicode only when necessary.
+            QTextCodec *latin1Codec = QTextCodec::codecForName("iso8859-1");
+            if (latin1Codec->canEncode(comment))
+            {
+                // write as ASCII
+                std::string exifComment("charset=\"Ascii\" ");
+                exifComment += comment.toLatin1().constData();
+                d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
+            }
+            else
+            {
+                // write as Unicode (UCS-2)
 
+                // Be aware that we are dealing with a UCS-2 string.
+                // Null termination means \0\0, strlen does not work,
+                // do not use any const-char*-only methods,
+                // pass a std::string and not a const char * to ExifDatum::operator=().
+                const unsigned short *ucs2 = comment.utf16();
+                std::string exifComment("charset=\"Unicode\" ");
+                exifComment.append((const char*)ucs2, sizeof(unsigned short) * comment.length());
+                d->exifMetadata["Exif.Photo.UserComment"] = exifComment;
+            }
+        }
         return true;
     }
     catch( Exiv2::Error &e )
@@ -558,7 +559,7 @@ bool KExiv2::getExifTagLong(const char* exifTagName, long &val) const
         Exiv2::ExifKey exifKey(exifTagName);
         Exiv2::ExifData exifData(d->exifMetadata);
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
-        if (it != exifData.end())
+        if (it != exifData.end() && it->count() > 0)
         {
             val = it->toLong();
             return true;
@@ -588,7 +589,8 @@ QByteArray KExiv2::getExifTagData(const char* exifTagName) const
 #else
             (*it).copy((Exiv2::byte*)s, exifData.byteOrder());
 #endif
-            QByteArray data = QByteArray::fromRawData(s, (*it).size());
+            QByteArray data(s, (*it).size());
+            delete s;
             return data;
         }
     }
@@ -685,10 +687,15 @@ QString KExiv2::getExifTagString(const char* exifTagName, bool escapeCR) const
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
+#if (EXIV2_TEST_VERSION(0,17,91))
+            // See B.K.O #184156 comment #13
+            std::string val  = it->print(&exifData);
+            QString tagValue = QString::fromLocal8Bit(val.c_str());
+#else
             std::ostringstream os;
             os << *it;
             QString tagValue = QString::fromLocal8Bit(os.str().c_str());
-
+#endif
             if (escapeCR)
                 tagValue.replace("\n", " ");
 
