@@ -7,28 +7,24 @@
  * Description : Exiv2 library interface for KDE
  *               Xmp manipulation methods
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
- *
- * NOTE: Do not use kdDebug() in this implementation because 
- *       it will be multithreaded. Use qDebug() instead. 
- *       See B.K.O #133026 for details.
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
  * Public License as published by the Free Software Foundation;
  * either version 2, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * ============================================================ */
 
 // Local includes.
 
-#include "kexiv2private.h"
+#include "kexiv2_p.h"
 #include "kexiv2.h"
 
 namespace KExiv2Iface
@@ -48,7 +44,8 @@ bool KExiv2::canWriteXmp(const QString& filePath)
     catch( Exiv2::Error &e )
     {
         std::string s(e.what());
-        qDebug("%s (Error #%i: %s)", "Cannot check Xmp access mode using Exiv2 ", e.code(), s.c_str());
+        kDebug(51003) << "Cannot check Xmp access mode using Exiv2 (Error #" 
+                      << e.code() << ": " << s.c_str() << ")" << endl;
     }
 
     return false;
@@ -110,7 +107,7 @@ QByteArray KExiv2::getXmp() const
     catch( Exiv2::Error &e )
     {
         if (!d->filePath.isEmpty())
-            qDebug ("From file %s", d->filePath.toAscii().constData());
+
 
         d->printExiv2ExceptionError("Cannot get Xmp data using Exiv2 ", e);
     }
@@ -140,7 +137,7 @@ bool KExiv2::setXmp(const QByteArray& data) const
     catch( Exiv2::Error &e )
     {
         if (!d->filePath.isEmpty())
-            qDebug ("From file %s", d->filePath.toAscii().constData());
+            kDebug(51003) << "From file " << d->filePath.toAscii().constData() << endl;
 
         d->printExiv2ExceptionError("Cannot set Xmp data using Exiv2 ", e);
     }
@@ -299,7 +296,7 @@ QString KExiv2::getXmpTagString(const char* xmpTagName, bool escapeCR) const
     catch( Exiv2::Error &e )
     {
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
-                                 .arg(xmpTagName), e);
+                                    .arg(xmpTagName), e);
     }
 
 #endif // _XMP_SUPPORT_
@@ -355,7 +352,7 @@ KExiv2::AltLangMap KExiv2::getXmpTagStringListLangAlt(const char* xmpTagName, bo
                     QString text = QString::fromUtf8(it2->second.c_str());
                     if (escapeCR)
                         text.replace("\n", " ");
-        
+
                     map.insert(lang, text);
                 }
 
@@ -366,7 +363,7 @@ KExiv2::AltLangMap KExiv2::getXmpTagStringListLangAlt(const char* xmpTagName, bo
     catch( Exiv2::Error &e )
     {
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
-                                 .arg(xmpTagName), e);
+                                    .arg(xmpTagName), e);
     }
 
 #endif // _XMP_SUPPORT_
@@ -387,19 +384,22 @@ bool KExiv2::setXmpTagStringListLangAlt(const char *xmpTagName, const KExiv2::Al
         // Remove old XMP alternative Language tag.
         removeXmpTag(xmpTagName);
 
-        Exiv2::Value::AutoPtr xmpTxtVal = Exiv2::Value::create(Exiv2::langAlt);
-        
-        for (AltLangMap::const_iterator it = values.begin(); it != values.end(); ++it)
+        if (!values.isEmpty())
         {
-            QString lang = it.key();  
-            QString text = it.value();
-            QString txtLangAlt = QString("lang=%1 %2").arg(lang).arg(text);
-            const std::string &txt(txtLangAlt.toUtf8().constData());
-            xmpTxtVal->read(txt);
-        }
+            Exiv2::Value::AutoPtr xmpTxtVal = Exiv2::Value::create(Exiv2::langAlt);
 
-        // ...and add the new one instead.
-        d->xmpMetadata.add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
+            for (AltLangMap::const_iterator it = values.begin(); it != values.end(); ++it)
+            {
+                QString lang = it.key();
+                QString text = it.value();
+                QString txtLangAlt = QString("lang=%1 %2").arg(lang).arg(text);
+                const std::string &txt(txtLangAlt.toUtf8().constData());
+                xmpTxtVal->read(txt);
+            }
+
+            // ...and add the new one instead.
+            d->xmpMetadata.add(Exiv2::XmpKey(xmpTagName), xmpTxtVal.get());
+        }
         return true;
     }
     catch( Exiv2::Error &e )
@@ -435,7 +435,7 @@ QString KExiv2::getXmpTagStringLangAlt(const char* xmpTagName, const QString& la
                     {
                         if (escapeCR)
                             tagValue.replace("\n", " ");
-            
+
                         return tagValue;
                     }
                 }
@@ -445,7 +445,7 @@ QString KExiv2::getXmpTagStringLangAlt(const char* xmpTagName, const QString& la
     catch( Exiv2::Error &e )
     {
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
-                                 .arg(xmpTagName), e);
+                                    .arg(xmpTagName), e);
     }
 
 #endif // _XMP_SUPPORT_
@@ -484,7 +484,7 @@ bool KExiv2::setXmpTagStringLangAlt(const char *xmpTagName, const QString& value
                 {
                     const std::string &val((*it).toUtf8().constData());
                     xmpTxtVal->read(val);
-                    qDebug() << *it << endl;
+                    kDebug(51003) << *it << endl;
                 }
             }
         }
@@ -524,13 +524,13 @@ QStringList KExiv2::getXmpTagStringSeq(const char* xmpTagName, bool escapeCR) co
                     std::ostringstream os;
                     os << it->toString(i);
                     QString seqValue = QString::fromUtf8(os.str().c_str());
-    
+
                     if (escapeCR)
                         seqValue.replace("\n", " ");
 
                     seq.append(seqValue);
                 }
-                qDebug() << "XMP String Seq (" << xmpTagName << "): " << seq << endl;  
+                kDebug(51003) << "XMP String Seq (" << xmpTagName << "): " << seq << endl;
 
                 return seq;
             }
@@ -539,7 +539,7 @@ QStringList KExiv2::getXmpTagStringSeq(const char* xmpTagName, bool escapeCR) co
     catch( Exiv2::Error &e )
     {
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
-                                 .arg(xmpTagName), e);
+                                    .arg(xmpTagName), e);
     }
 
 #endif // _XMP_SUPPORT_
@@ -557,16 +557,23 @@ bool KExiv2::setXmpTagStringSeq(const char *xmpTagName, const QStringList& seq,
 
     try
     {
-        QStringList list = seq;
-        Exiv2::Value::AutoPtr xmpTxtSeq = Exiv2::Value::create(Exiv2::xmpSeq);
-
-        for (QStringList::Iterator it = list.begin(); it != list.end(); ++it )
+        if (seq.isEmpty())
         {
-            const std::string &txt((*it).toUtf8().constData());
-            xmpTxtSeq->read(txt);
+            removeXmpTag(xmpTagName);
+        }
+        else
+        {
+            const QStringList list = seq;
+            Exiv2::Value::AutoPtr xmpTxtSeq = Exiv2::Value::create(Exiv2::xmpSeq);
+
+            for (QStringList::const_iterator it = list.constBegin(); it != list.constEnd(); ++it )
+            {
+                const std::string &txt((*it).toUtf8().constData());
+                xmpTxtSeq->read(txt);
+            }
+            d->xmpMetadata[xmpTagName].setValue(xmpTxtSeq.get());
+            return true;
         }
-        d->xmpMetadata.add(Exiv2::XmpKey(xmpTagName), xmpTxtSeq.get());
-        return true;
     }
     catch( Exiv2::Error &e )
     {
@@ -598,7 +605,7 @@ QStringList KExiv2::getXmpTagStringBag(const char* xmpTagName, bool escapeCR) co
                     std::ostringstream os;
                     os << it->toString(i);
                     QString bagValue = QString::fromUtf8(os.str().c_str());
-    
+
                     if (escapeCR)
                         bagValue.replace("\n", " ");
 
@@ -612,7 +619,7 @@ QStringList KExiv2::getXmpTagStringBag(const char* xmpTagName, bool escapeCR) co
     catch( Exiv2::Error &e )
     {
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
-                                 .arg(xmpTagName), e);
+                                    .arg(xmpTagName), e);
     }
 
 #endif // _XMP_SUPPORT_
@@ -630,16 +637,23 @@ bool KExiv2::setXmpTagStringBag(const char *xmpTagName, const QStringList& bag,
 
     try
     {
-        QStringList list = bag;
-        Exiv2::Value::AutoPtr xmpTxtBag = Exiv2::Value::create(Exiv2::xmpBag);
-
-        for (QStringList::Iterator it = list.begin(); it != list.end(); ++it )
+        if (bag.isEmpty())
         {
-            const std::string &txt((*it).toUtf8().constData());
-            xmpTxtBag->read(txt);
+            removeXmpTag(xmpTagName);
+        }
+        else
+        {
+            QStringList list = bag;
+            Exiv2::Value::AutoPtr xmpTxtBag = Exiv2::Value::create(Exiv2::xmpBag);
+
+            for (QStringList::const_iterator it = list.constBegin(); it != list.constEnd(); ++it )
+            {
+                const std::string &txt((*it).toUtf8().constData());
+                xmpTxtBag->read(txt);
+            }
+            d->xmpMetadata[xmpTagName].setValue(xmpTxtBag.get());
+            return true;
         }
-        d->xmpMetadata.add(Exiv2::XmpKey(xmpTagName), xmpTxtBag.get());
-        return true;
     }
     catch( Exiv2::Error &e )
     {
@@ -719,6 +733,7 @@ QVariant KExiv2::getXmpTagVariant(const char *xmpTagName, bool rationalAsListOfI
                 }
                 case Exiv2::xmpBag:
                 case Exiv2::xmpSeq:
+                case Exiv2::xmpAlt:
                 {
                     QStringList list;
                     for (int i=0; i < it->count(); i++)
@@ -727,7 +742,7 @@ QVariant KExiv2::getXmpTagVariant(const char *xmpTagName, bool rationalAsListOfI
                     }
                     return list;
                 }
-                case Exiv2::xmpAlt:
+                case Exiv2::langAlt:
                 {
                     // access the value directly
                     const Exiv2::LangAltValue &value = static_cast<const Exiv2::LangAltValue &>(it->value());
@@ -748,7 +763,7 @@ QVariant KExiv2::getXmpTagVariant(const char *xmpTagName, bool rationalAsListOfI
     catch( Exiv2::Error &e )
     {
         d->printExiv2ExceptionError(QString("Cannot find Xmp key '%1' into image using Exiv2 ")
-                                 .arg(xmpTagName), e);
+                                    .arg(xmpTagName), e);
     }
 
 #endif // _XMP_SUPPORT_
@@ -793,11 +808,13 @@ bool KExiv2::removeXmpTag(const char *xmpTagName, bool setProgramName) const
             d->xmpMetadata.erase(it);
             return true;
         }
+
+        return true;
     }
     catch( Exiv2::Error &e )
     {
         d->printExiv2ExceptionError("Cannot remove Xmp tag using Exiv2 ", e);
-    }        
+    }
 
 #endif // _XMP_SUPPORT_
 
@@ -818,16 +835,16 @@ bool KExiv2::setXmpKeywords(const QStringList& newKeywords, bool setProgramName)
 
     QStringList oldkeys = getXmpKeywords();
     QStringList newkeys = newKeywords;
-    
-    // Create a list of keywords including old one witch already exists.
-    for (QStringList::Iterator it = oldkeys.begin(); it != oldkeys.end(); ++it )
+
+    // Create a list of keywords including old one which already exists.
+    for (QStringList::const_iterator it = oldkeys.constBegin(); it != oldkeys.constEnd(); ++it )
     {
         if (!newkeys.contains(*it))
             newkeys.append(*it);
     }
 
     if (setXmpTagStringBag("Xmp.dc.subject", newkeys, false))
-        return false;
+        return true;
 
 #endif // _XMP_SUPPORT_
 
@@ -846,18 +863,18 @@ bool KExiv2::setXmpSubCategories(const QStringList& newSubCategories, bool setPr
     if (!setProgramId(setProgramName))
         return false;
 
-    QStringList oldSubCat = getXmpSubCategories();
+    const QStringList oldSubCat = getXmpSubCategories();
     QStringList newSubCat = newSubCategories;
-    
-    // Create a list of sub-categories including old one witch already exists.
-    for (QStringList::Iterator it = oldSubCat.begin(); it != oldSubCat.end(); ++it )
+
+    // Create a list of sub-categories including old one which already exists.
+    for (QStringList::const_iterator it = oldSubCat.constBegin(); it != oldSubCat.constEnd(); ++it )
     {
         if (!newSubCat.contains(*it))
             newSubCat.append(*it);
     }
 
     if (setXmpTagStringBag("Xmp.photoshop.SupplementalCategories", newSubCat, false))
-        return false;
+        return true;
 
 #endif // _XMP_SUPPORT_
 
@@ -878,16 +895,16 @@ bool KExiv2::setXmpSubjects(const QStringList& newSubjects, bool setProgramName)
 
     QStringList oldSubjectCodes = getXmpSubjects();
     QStringList newSubjectCodes = newSubjects;
-    
-    // Create a list of sub-categories including old one witch already exists.
-    for (QStringList::Iterator it = oldSubjectCodes.begin(); it != oldSubjectCodes.end(); ++it )
+
+    // Create a list of sub-categories including old one which already exists.
+    for (QStringList::const_iterator it = oldSubjectCodes.constBegin(); it != oldSubjectCodes.constEnd(); ++it )
     {
         if (!newSubjectCodes.contains(*it))
             newSubjectCodes.append(*it);
     }
 
     if (setXmpTagStringBag("Xmp.iptc.SubjectCode", newSubjectCodes, false))
-        return false;
+        return true;
 
 #endif // _XMP_SUPPORT_
 
