@@ -18,6 +18,7 @@
 #include <qlayout.h>
 #include <qmutex.h>
 #include <qregexp.h>
+#include <qstack.h>
 #include <qtextstream.h>
 #include <QtGui/QPrinter>
 
@@ -170,9 +171,25 @@ Okular::Action* createLinkFromPopplerLink(const Poppler::Link *popplerLink)
 		break;
 	
 		case Poppler::Link::Goto:
+		{
 			popplerLinkGoto = static_cast<const Poppler::LinkGoto *>(popplerLink);
+#ifdef HAVE_POPPLER_0_11
+			const Poppler::LinkDestination dest = popplerLinkGoto->destination();
+			const QString destName = dest.destinationName();
+			if (destName.isEmpty())
+			{
+				fillViewportFromLinkDestination( viewport, dest );
+				link = new Okular::GotoAction(popplerLinkGoto->fileName(), viewport);
+			}
+			else
+			{
+				link = new Okular::GotoAction(popplerLinkGoto->fileName(), destName);
+			}
+#else
 			fillViewportFromLinkDestination( viewport, popplerLinkGoto->destination() );
 			link = new Okular::GotoAction(popplerLinkGoto->fileName(), viewport);
+#endif
+		}
 		break;
 		
 		case Poppler::Link::Execute:
@@ -258,7 +275,7 @@ static KAboutData createAboutData()
          "okular_poppler",
          "okular_poppler",
          ki18n( "PDF Backend" ),
-         "0.2",
+         "0.2.1",
          ki18n( "A PDF file renderer" ),
          KAboutData::License_GPL,
          ki18n( "© 2005-2008 Albert Astals Cid" )
@@ -272,7 +289,8 @@ OKULAR_EXPORT_PLUGIN(PDFGenerator, createAboutData())
 PDFGenerator::PDFGenerator( QObject *parent, const QVariantList &args )
     : Generator( parent, args ), pdfdoc( 0 ), ready( true ),
     pixmapRequest( 0 ), docInfoDirty( true ), docSynopsisDirty( true ),
-    docEmbeddedFilesDirty( true ), nextFontPage( 0 )
+    docEmbeddedFilesDirty( true ), nextFontPage( 0 ),
+    dpiX( Okular::Utils::dpiX() ), dpiY( Okular::Utils::dpiY() )
 {
     setFeature( TextExtraction );
     setFeature( FontInfo );
@@ -430,6 +448,7 @@ bool PDFGenerator::doCloseDocument()
     qDeleteAll(docEmbeddedFiles);
     docEmbeddedFiles.clear();
     nextFontPage = 0;
+    rectsGenerated.clear();
 
     return true;
 }
@@ -437,14 +456,15 @@ bool PDFGenerator::doCloseDocument()
 void PDFGenerator::loadPages(QVector<Okular::Page*> &pagesVector, int rotation, bool clear)
 {
     // TODO XPDF 3.01 check
-    int count=pagesVector.count(),w=0,h=0;
+    const int count = pagesVector.count();
+    double w = 0, h = 0;
     for ( int i = 0; i < count ; i++ )
     {
         // get xpdf page
         Poppler::Page * p = pdfdoc->page( i );
-        QSize pSize = p->pageSize();
-        w = pSize.width();
-        h = pSize.height();
+        const QSizeF pSize = p->pageSizeF();
+        w = pSize.width() / 72.0 * dpiX;
+        h = pSize.height() / 72.0 * dpiY;
         Okular::Rotation orientation = Okular::Rotation0;
         switch (p->orientation())
         {
@@ -513,7 +533,7 @@ const Okular::DocumentInfo * PDFGenerator::generateDocumentInfo()
                          KGlobal::locale()->formatDateTime( pdfdoc->date("ModDate"), KLocale::LongDate, true ) );
 
             docInfo.set( "format", i18nc( "PDF v. <version>", "PDF v. %1",
-                          pdfdoc->pdfVersion() ), i18n( "Format" ) );
+                         QString::number( pdfdoc->pdfVersion() ) ), i18n( "Format" ) );
             docInfo.set( "encryption", pdfdoc->isEncrypted() ? i18n( "Encrypted" ) : i18n( "Unencrypted" ),
                          i18n("Security") );
             docInfo.set( "optimization", pdfdoc->isLinearized() ? i18n( "Yes" ) : i18n( "No" ),
@@ -743,8 +763,8 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     if ( page->rotation() % 2 )
         qSwap( pageWidth, pageHeight );
 
-    double fakeDpiX = request->width() * 72.0 / pageWidth,
-           fakeDpiY = request->height() * 72.0 / pageHeight;
+    double fakeDpiX = request->width() * dpiX / pageWidth,
+           fakeDpiY = request->height() * dpiY / pageHeight;
 
     // setup Okular:: output device: text page is generated only if we are at 72dpi.
     // since we can pre-generate the TextPage at the right res.. why not?
@@ -782,12 +802,9 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     userMutex()->unlock();
     if ( genTextPage )
     {
-#ifdef HAVE_POPPLER_0_7
         QList<Poppler::TextBox*> textList = p->textList();
-#else
-        QList<Poppler::TextBox*> textList = p->textList((Poppler::Page::Rotation)request->page()->orientation());
-#endif
-        Okular::TextPage *tp = abstractTextPage(textList, page->height(), page->width(), request->page()->orientation());
+        const QSizeF s = p->pageSizeF();
+        Okular::TextPage *tp = abstractTextPage(textList, s.height(), s.width(), request->page()->orientation());
         page->setTextPage( tp );
         qDeleteAll(textList);
         
@@ -805,25 +822,20 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
 
 Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
 {
-    kDebug(PDFDebug) << "calling" ;
+#ifdef PDFGENERATOR_DEBUG
+    kDebug(PDFDebug) << "page" << page->number();
+#endif
     // build a TextList...
     Poppler::Page *pp = pdfdoc->page( page->number() );
     userMutex()->lock();
-#ifdef HAVE_POPPLER_0_7
     QList<Poppler::TextBox*> textList = pp->textList();
-#else
-    QList<Poppler::TextBox*> textList = pp->textList((Poppler::Page::Rotation)page->orientation());
-#endif
     userMutex()->unlock();
-    delete pp;
 
-#ifdef HAVE_POPPLER_0_7
-    const double pageWidth = page->width();
-    const double pageHeight = page->height();
-#else
-    const double pageWidth = ( page->rotation() % 2 ? page->height() : page->width() );
-    const double pageHeight = ( page->rotation() % 2 ? page->width() : page->height() );
-#endif
+    QSizeF s = pp->pageSizeF();
+    const double pageWidth = s.width();
+    const double pageHeight = s.height();
+
+    delete pp;
 
     Okular::TextPage *tp = abstractTextPage(textList, pageHeight, pageWidth, (Poppler::Page::Rotation)page->orientation());
     qDeleteAll(textList);
@@ -841,8 +853,10 @@ void PDFGenerator::requestFontData(const Okular::FontInfo &font, QByteArray *dat
 #endif
 }
 
+#define DUMMY_QPRINTER_COPY
 bool PDFGenerator::print( QPrinter& printer )
 {
+#ifdef DUMMY_QPRINTER_COPY
     // Get the real page size to pass to the ps generator
     QPrinter dummy( QPrinter::PrinterResolution );
     dummy.setFullPage( true );
@@ -851,6 +865,10 @@ bool PDFGenerator::print( QPrinter& printer )
     dummy.setPaperSize( printer.paperSize( QPrinter::Millimeter ), QPrinter::Millimeter );
     int width = dummy.width();
     int height = dummy.height();
+#else
+    int width = printer.width();
+    int height = printer.height();
+#endif
 
     // Create the tempfile to send to FilePrinter, which will manage the deletion
     KTemporaryFile tf;
@@ -865,11 +883,7 @@ bool PDFGenerator::print( QPrinter& printer )
 
     // TODO rotation
 
-#ifdef HAVE_POPPLER_0_7
     tf.setAutoRemove(false);
-#else
-    tf.close();
-#endif
 
     QString pstitle = metaData(QLatin1String("Title"), QVariant()).toString();
     if ( pstitle.trimmed().isEmpty() )
@@ -885,11 +899,7 @@ bool PDFGenerator::print( QPrinter& printer )
 
     Poppler::PSConverter *psConverter = pdfdoc->psConverter();
 
-#ifdef HAVE_POPPLER_0_7
     psConverter->setOutputDevice(&tf);
-#else
-    psConverter->setOutputFileName(tempfilename);
-#endif
 
     psConverter->setPageList(pageList);
     psConverter->setPaperWidth(width);
@@ -907,9 +917,7 @@ bool PDFGenerator::print( QPrinter& printer )
     {
         userMutex()->unlock();
         delete psConverter;
-#ifdef HAVE_POPPLER_0_7
         tf.close();
-#endif
         int ret = Okular::FilePrinter::printFile( printer, tempfilename,
                                                   Okular::FilePrinter::SystemDeletesFiles,
                                                   Okular::FilePrinter::ApplicationSelectsPages,
@@ -1078,11 +1086,12 @@ Okular::TextPage * PDFGenerator::abstractTextPage(const QList<Poppler::TextBox*>
 {    
     Okular::TextPage* ktp=new Okular::TextPage;
     Poppler::TextBox *next; 
-    kWarning(PDFDebug) << "getting text page in generator pdf - rotation:" << rot;
+#ifdef PDFGENERATOR_DEBUG
+    kDebug(PDFDebug) << "getting text page in generator pdf - rotation:" << rot;
+#endif
     int charCount=0;
     int j;
     QString s;
-#ifdef HAVE_POPPLER_0_7
     foreach (Poppler::TextBox *word, text)
     {
         charCount=word->text().length();
@@ -1113,117 +1122,6 @@ Okular::TextPage * PDFGenerator::abstractTextPage(const QList<Poppler::TextBox*>
                      wordBBox.top()/height);
         }
     }
-#else
-    Okular::NormalizedRect * wordRect = new Okular::NormalizedRect;
-    
-    rot = rot % 4;
-    
-    foreach (Poppler::TextBox *word, text)
-    {
-        wordRect->left = word->boundingBox().left();
-        wordRect->bottom = word->boundingBox().bottom();
-        wordRect->right = word->boundingBox().right();
-        wordRect->top = word->boundingBox().top();
-        charCount=word->text().length();
-        next=word->nextWord();
-        switch (rot)
-        {
-            case 0:
-            // 0 degrees, normal word boundaries are top and bottom
-            // only x boundaries change the order of letters is normal not reversed
-            for (j = 0; j < charCount; j++)
-            {
-                s = word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    // this letters boundary
-                    word->edge(j)/width,
-                    wordRect->bottom/height,
-                    // next letters boundary
-                    word->edge(j+1)/width,
-                    wordRect->top/height);
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     word->edge(charCount)/width,
-                     wordRect->bottom/height,
-                    // next letters boundary
-                     next->edge(0)/width,
-                     wordRect->top/height);
-            break;
-
-            case 1:
-            // 90 degrees, x boundaries not changed
-            // y ones change, the order of letters is normal not reversed
-            for (j=0;j<charCount;j++)
-            {
-                s=word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    wordRect->left/width,
-                    word->edge(j)/height,
-                    wordRect->right/width,
-                    word->edge(j+1)/height);
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     wordRect->left/width,
-                     word->edge(charCount)/height,
-                    // next letters boundary
-                     wordRect->right/width,
-                     next->edge(0)/height);
-            break;
-
-            case 2:
-            // same as case 0 but reversed order of letters
-            for (j=0;j<charCount;j++)
-            {
-                s=word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    word->edge(j+1)/width,
-                    wordRect->bottom/height,
-                    word->edge(j)/width,
-                    wordRect->top/height);
-
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     next->edge(0)/width,
-                     wordRect->bottom/height,
-                    // next letters boundary
-                     word->edge(charCount)/width,
-                     wordRect->top/height);
-           
-            break;
-
-            case 3:
-            for (j=0;j<charCount;j++)
-            {
-                s=word->text().at(j);
-                append(ktp, (j==charCount-1 && !next ) ? (s + '\n') : s,
-                    wordRect->left/width,
-                    word->edge(j+1)/height,
-                    wordRect->right/width,
-                    word->edge(j)/height);
-            }
-            
-            if ( word->hasSpaceAfter() && next )
-              append(ktp, " ",
-                    // this letters boundary
-                     wordRect->left/width,
-                     next->edge(0)/height,
-                    // next letters boundary
-                     wordRect->right/width,
-                     word->edge(charCount)/height);
-            break;
-        }
-    }
-    delete wordRect;
-#endif
     return ktp;
 }
 
@@ -1396,11 +1294,9 @@ void PDFGenerator::addFormFields( Poppler::Page * popplerPage, Okular::Page * pa
         Okular::FormField * of = 0;
         switch ( f->type() )
         {
-#ifdef HAVE_POPPLER_0_7
             case Poppler::FormField::FormButton:
                 of = new PopplerFormFieldButton( static_cast<Poppler::FormFieldButton*>( f ) );
                 break;
-#endif
             case Poppler::FormField::FormText:
                 of = new PopplerFormFieldText( static_cast<Poppler::FormFieldText*>( f ) );
                 break;
@@ -1447,13 +1343,16 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
         return;
 
     QHash<int, pdfsyncpoint> points;
-    QString currentfile;
+    QStack<QString> fileStack;
     int currentpage = -1;
     QRegExp newfilere( "\\(\\s*([^\\s]+)" );
     QRegExp linere( "l\\s+(\\d+)\\s+(\\d+)(\\s+(\\d+))?" );
     QRegExp pagere( "s\\s+(\\d+)" );
     QRegExp locre( "p\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
     QRegExp locstarre( "p\\*\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)" );
+    const QLatin1String texStr( ".tex" );
+
+    fileStack.push( coreName + texStr );
 
     QString line;
     while ( !ts.atEnd() )
@@ -1471,7 +1370,7 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
                 pt.row = linere.cap( 2 ).toInt();
                 pt.column = 0; // TODO
                 pt.page = -1;
-                pt.file = currentfile;
+                pt.file = fileStack.top();
                 points[ id ] = pt;
             }
         }
@@ -1498,21 +1397,20 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
         else if ( line.startsWith( QLatin1Char( '(' ) ) && newfilere.exactMatch( line ) )
         {
             QString newfile = newfilere.cap( 1 );
-            if ( currentfile.isEmpty() )
+            if ( !newfile.endsWith( texStr ) )
             {
-                currentfile = newfile;
+                newfile += texStr;
             }
-            else
-                kDebug(PDFDebug) << "PdfSync: more than one file level:" << newfile;
+            fileStack.push( newfile );
         }
         else if ( line == QLatin1String( ")" ) )
         {
-            if ( !currentfile.isEmpty() )
+            if ( !fileStack.isEmpty() )
             {
-                currentfile.clear();
+                fileStack.pop();
             }
             else
-                kDebug(PDFDebug) << "PdfSync: going one level down:" << currentfile;
+                kDebug(PDFDebug) << "PdfSync: going one level down too much";
         }
         else
             kDebug(PDFDebug).nospace() << "PdfSync: unknown line format: '" << line << "'";
@@ -1528,22 +1426,10 @@ void PDFGenerator::loadPdfSync( const QString & filePath, QVector<Okular::Page*>
 
         // maginc numbers for TeX's RSU's (Ridiculously Small Units) conversion to pixels
         Okular::NormalizedPoint p(
-            ( pt.x * 72.0 ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->width() ),
-            ( pt.y * 72.0 ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->height() )
+            ( pt.x * dpiX ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->width() ),
+            ( pt.y * dpiY ) / ( 72.27 * 65536.0 * pagesVector[pt.page]->height() )
             );
-        QString file;
-        if ( !pt.file.isEmpty() )
-        {
-            file = pt.file;
-            int dotpos = file.lastIndexOf( QLatin1Char( '.' ) );
-            QString ext;
-            if ( dotpos == -1 )
-                file += QString::fromLatin1( ".tex" );
-        }
-        else
-        {
-            file = coreName + QLatin1String( ".tex" );
-        }
+        QString file = pt.file;
         Okular::SourceReference * sourceRef = new Okular::SourceReference( file, pt.row, pt.column );
         refRects[ pt.page ].append( new Okular::SourceRefObjectRect( p, sourceRef ) );
     }
@@ -1565,10 +1451,8 @@ bool PDFGenerator::supportsOption( SaveOption option ) const
 {
     switch ( option )
     {
-#ifdef HAVE_POPPLER_0_7
         case SaveChanges:
             return true;
-#endif
         default: ;
     }
     return false;
@@ -1576,7 +1460,6 @@ bool PDFGenerator::supportsOption( SaveOption option ) const
 
 bool PDFGenerator::save( const QString &fileName, SaveOptions options )
 {
-#ifdef HAVE_POPPLER_0_7
     Poppler::PDFConverter *pdfConv = pdfdoc->pdfConverter();
 
     pdfConv->setOutputFileName( fileName );
@@ -1587,11 +1470,6 @@ bool PDFGenerator::save( const QString &fileName, SaveOptions options )
     bool success = pdfConv->convert();
     delete pdfConv;
     return success;
-#else
-    Q_UNUSED( fileName )
-    Q_UNUSED( options )
-    return false;
-#endif
 }
 
 void PDFGenerator::threadFinished()
@@ -1637,10 +1515,13 @@ void PDFGenerator::threadFinished()
     delete outImage;
     if ( !outText.isEmpty() )
     {
-        Okular::TextPage *tp = abstractTextPage( outText, request->page()->height(), 
-                                                 request->page()->width(),request->page()->orientation());
+        Poppler::Page *pp = pdfdoc->page( request->page()->number() );
+        const QSizeF s = pp->pageSizeF();
+        Okular::TextPage *tp = abstractTextPage( outText, s.height(), 
+                                                 s.width(),request->page()->orientation());
         request->page()->setTextPage( tp );
         qDeleteAll(outText);
+        delete pp;
         
         // notify the new generation
         signalTextGenerationDone( request->page(), tp );
@@ -1805,10 +1686,9 @@ void PDFPixmapGeneratorThread::run()
 
     // 1. set OutputDev parameters and Generate contents
     Poppler::Page *pp = d->generator->pdfdoc->page( page->number() );
-    const QSizeF &pageSizeF = pp->pageSizeF();
     
-    double fakeDpiX = width * 72.0 / pageSizeF.width(),
-           fakeDpiY = height * 72.0 / pageSizeF.height();
+    double fakeDpiX = width * d->generator->dpiX / page->width(),
+           fakeDpiY = height * d->generator->dpiY / page->height();
 
     // 2. grab data from the OutputDev and store it locally (note takeIMAGE)
 #ifndef NDEBUG
@@ -1827,11 +1707,7 @@ void PDFPixmapGeneratorThread::run()
 
     if ( genTextPage )
     {
-#ifdef HAVE_POPPLER_0_7
         d->m_textList = pp->textList();
-#else
-        d->m_textList = pp->textList((Poppler::Page::Rotation)d->currentRequest->page()->orientation());
-#endif
     }
     delete pp;
     
